library free_list;

import 'dart:collection';

class FreeList<T> {

  /**
   * Constructor closure for creating new instances
   */
  Function     _constructor;

  /**
   * The internal free list
   */
  ListQueue<T> _queue;

  /**
   * Maximum elements to store in the free list
   */
  int          _maxSize;

  /**
   * Current size of the free list
   */
  int          _currentSize;

  /**
   * The free list constructor which takes a closure constuctor
   * responsible for allocating objects for the free list
   *
   * @param {T()} constructor - The closure constructor
   */
  FreeList(T constructor()) {
    this._constructor = constructor;
    this._currentSize = 0;
    this._maxSize     = 100;
    this._queue       = new ListQueue<T>();
  }

  /**
   * A specialization of the constructor where you can manually
   * specify the maximum size of the free list
   *
   * @param {T()} constructor - The closure constructor
   * @param {int} max         - The max size of the free list
   */
  FreeList.max(T constructor(), int max) {
    this._constructor = constructor;
    this._currentSize = 0;
    this._maxSize     = max;
    this._queue       = new ListQueue<T>();
  }

  /**
   * If there are available resources in the free list, returns the
   * hottest one, otherwise allocate and return a new one.
   *
   * @return {T} - An instance of the allocated type
   */
  T acquire() {
    if (size > 0) {
      this._currentSize--;
      return this._queue.removeFirst();
    } else {
      return this._constructor();
    }
  }

  /**
   * Add a no longer used resource to the free list to be used by
   * another computation
   *
   * @param {T} value - The value to return to the free list
   * @return {void}
   */
  void release(T value) {
    if (this._currentSize < this._maxSize) {
      this._currentSize++;
    } else {
      this._queue.removeLast();
    }
    this._queue.addFirst(value);
  }

  /**
   * The current size of the free list
   *
   * @return {int} - Size
   */
  int get size  => this._currentSize;

  /**
   * The max size of the free list
   *
   * @return {int} - Max size
   */
  int get maxSize => this._maxSize;

}